@@ -477,9 +477,8 @@ def sana12(M1, M2, a, P, amin, amax, x0, x1, p):
 
     arguments are M1, M2, a, Period P, amin, amax, x0=log P0, x1=log P1, p
 
-    example args: 10, 5, ?, P, ?, ?, -2, 12, -0.55
+    example args: 10, 5, sep(M1, M2, P), sep, ?, -2, 12, -0.55
 
-    # TODO: ? wtf. vague input
     # TODO: Fix this function!
     """
 
@@ -496,8 +495,8 @@ def sana12(M1, M2, a, P, amin, amax, x0, x1, p):
         xmin = LOG_LN_CONVERTER * math.log(calc_period_from_sep(M1, M2, amin))
         xmax = LOG_LN_CONVERTER * math.log(calc_period_from_sep(M1, M2, amax))
 
-        print("M1 M2 amin amax P x xmin xmax")
-        print(M1, M2, amin, amax, P, x, xmin, xmax)
+        # print("M1 M2 amin amax P x xmin xmax")
+        # print(M1, M2, amin, amax, P, x, xmin, xmax)
         # my $x0 = 0.15;
         # my $x1 = 3.5;
 
@@ -519,7 +518,7 @@ def sana12(M1, M2, a, P, amin, amax, x0, x1, p):
 
 def Izzard2012_period_distribution(P, M1, log10Pmin=1):
     """
-    period distribution which interpolates between 
+    period distribution which interpolates between
     Duquennoy and Mayor 1991 at low mass (G/K spectral type <~1.15Msun)
     and Sana et al 2012 at high mass (O spectral type >~16.3Msun)
 
@@ -598,7 +597,7 @@ def Izzard2012_period_distribution(P, M1, log10Pmin=1):
 def interpolate_in_mass_izzard2012(M, high, low):
     """
     Function to interpolate in mass
-    
+
     high: at M=16.3
     low: at 1.15
     """
@@ -626,8 +625,6 @@ def flatsections(x, opts):
 
     opts: list of dicts with settings for the flat sections
     x: location to calculate the y value
-
-    TODO: figure out why it has to be a list of dict. why not just 1
     """
 
     c = 0
