@@ -163,7 +163,7 @@ def parse_binary_c_version_info(version_info_string):
     ##########################
     # Isotopes:
     # Split off
-    isotopes = set([el for el in cleaned if el.startswith('Isotope ')])
+    isotopes = set([el for el in cleaned if el.startswith("Isotope ")])
     cleaned = cleaned - isotopes
 
     isotope_dict = {}
@@ -171,23 +171,36 @@ def parse_binary_c_version_info(version_info_string):
         split_info = el.split("Isotope ")[-1].strip().split(" is ")
 
         isotope_info = split_info[-1]
-        name = isotope_info.split(' ')[0].strip()
+        name = isotope_info.split(" ")[0].strip()
 
         # Get details
-        mass_g = float(isotope_info.split(",")[0].split('(')[1].split("=")[-1][:-2].strip())
-        mass_amu = float(isotope_info.split(",")[0].split('(')[-1].split("=")[-1].strip())
-        mass_mev = float(isotope_info.split(",")[-3].split("=")[-1].replace(")", "").strip())
+        mass_g = float(
+            isotope_info.split(",")[0].split("(")[1].split("=")[-1][:-2].strip()
+        )
+        mass_amu = float(
+            isotope_info.split(",")[0].split("(")[-1].split("=")[-1].strip()
+        )
+        mass_mev = float(
+            isotope_info.split(",")[-3].split("=")[-1].replace(")", "").strip()
+        )
         A = int(isotope_info.split(",")[-1].strip().split("=")[-1].replace(")", ""))
         Z = int(isotope_info.split(",")[-2].strip().split("=")[-1])
 
         #
-        isotope_dict[int(split_info[0])] = {'name': name, 'Z': Z, 'A': A, 'mass_mev': mass_mev, 'mass_g': mass_g, 'mass_amu': mass_amu}
-    version_info_dict['isotopes'] = isotope_dict
+        isotope_dict[int(split_info[0])] = {
+            "name": name,
+            "Z": Z,
+            "A": A,
+            "mass_mev": mass_mev,
+            "mass_g": mass_g,
+            "mass_amu": mass_amu,
+        }
+    version_info_dict["isotopes"] = isotope_dict if isotope_dict else None
 
     ##########################
     # Argpairs:
     # Split off
-    argpairs = set([el for el in cleaned if el.startswith('ArgPair')])
+    argpairs = set([el for el in cleaned if el.startswith("ArgPair")])
     cleaned = cleaned - argpairs
 
     argpair_dict = {}
@@ -199,33 +212,33 @@ def parse_binary_c_version_info(version_info_string):
         else:
             argpair_dict[split_info[0]][split_info[1]] = split_info[2]
 
-    version_info_dict['argpairs'] = argpair_dict
+    version_info_dict["argpairs"] = argpair_dict if argpair_dict else None
 
     ##########################
     # ensembles:
     # Split off
-    ensembles = set([el for el in cleaned if el.startswith('Ensemble')])
+    ensembles = set([el for el in cleaned if el.startswith("Ensemble")])
     cleaned = cleaned - ensembles
 
     ensemble_dict = {}
     for el in ensembles:
         split_info = el.split("Ensemble ")[-1].split(" is ")
-        if len(split_info)>1:
+        if len(split_info) > 1:
             ensemble_dict[int(split_info[0])] = split_info[-1]
-    version_info_dict['ensembles'] = ensemble_dict
+    version_info_dict["ensembles"] = ensemble_dict if ensemble_dict else None
 
     ##########################
     # macros:
     # Split off
-    macros = set([el for el in cleaned if el.startswith('macroxyz')])
+    macros = set([el for el in cleaned if el.startswith("macroxyz")])
     cleaned = cleaned - macros
 
     param_type_dict = {
-        'STRING': str,
-        'FLOAT': float,
-        'MACRO': str,
-        'INT': int,
-        'LONG_INT': int,
+        "STRING": str,
+        "FLOAT": float,
+        "MACRO": str,
+        "INT": int,
+        "LONG_INT": int,
     }
 
     macros_dict = {}
@@ -241,12 +254,12 @@ def parse_binary_c_version_info(version_info_string):
             macros_dict[param_name] = param_type_dict[param_type](param_value)
         except ValueError:
             macros_dict[param_name] = str(param_value)
-    version_info_dict['macros'] = macros_dict
+    version_info_dict["macros"] = macros_dict if macros_dict else None
 
     ##########################
     # Elements:
     # Split off:
-    elements = set([el for el in cleaned if el.startswith('Element')])
+    elements = set([el for el in cleaned if el.startswith("Element")])
     cleaned = cleaned - elements
 
     # Fill dict:
@@ -257,31 +270,42 @@ def parse_binary_c_version_info(version_info_string):
 
         # get isotope info
         isotopes = {}
-        if not split_info[-1][0]=='0':
+        if not split_info[-1][0] == "0":
             isotope_string = split_info[-1].split(" = ")[-1]
-            isotopes = {int(split_isotope.split("=")[0]):split_isotope.split("=")[1] for split_isotope in isotope_string.split(" ")}
+            isotopes = {
+                int(split_isotope.split("=")[0]): split_isotope.split("=")[1]
+                for split_isotope in isotope_string.split(" ")
+            }
 
-        elements_dict[int(name_info[0])] = {'name': name_info[-1], 'atomic_number': int(name_info[0]), 'amt_isotopes': len(isotopes), 'isotopes': isotopes}
-    version_info_dict['elements'] = version_info_dict
+        elements_dict[int(name_info[0])] = {
+            "name": name_info[-1],
+            "atomic_number": int(name_info[0]),
+            "amt_isotopes": len(isotopes),
+            "isotopes": isotopes,
+        }
+    version_info_dict["elements"] = elements_dict if elements_dict else None
 
     ##########################
     # dt_limits:
     # split off
-    dt_limits = set([el for el in cleaned if el.startswith('DTlimit')])
+    dt_limits = set([el for el in cleaned if el.startswith("DTlimit")])
     cleaned = cleaned - dt_limits
 
     # Fill dict
     dt_limits_dict = {}
     for el in dt_limits:
         split_info = el.split("DTlimit ")[-1].split(" : ")
-        dt_limits_dict[split_info[1].strip()] = {'index': int(split_info[0]), 'value': float(split_info[-1])}
+        dt_limits_dict[split_info[1].strip()] = {
+            "index": int(split_info[0]),
+            "value": float(split_info[-1]),
+        }
 
-    version_info_dict['dt_limits'] = dt_limits_dict
+    version_info_dict["dt_limits"] = dt_limits_dict if dt_limits_dict else None
 
     ##########################
     # Nucleosynthesis sources:
     # Split off
-    nucsyn_sources = set([el for el in cleaned if el.startswith('Nucleosynthesis')])
+    nucsyn_sources = set([el for el in cleaned if el.startswith("Nucleosynthesis")])
     cleaned = cleaned - nucsyn_sources
 
     # Fill dict
@@ -290,7 +314,9 @@ def parse_binary_c_version_info(version_info_string):
         split_info = el.split("Nucleosynthesis source")[-1].strip().split(" is ")
         nucsyn_sources_dict[int(split_info[0])] = split_info[-1]
 
-    version_info_dict['nucleosynthesis_sources'] = nucsyn_sources_dict
+    version_info_dict["nucleosynthesis_sources"] = (
+        nucsyn_sources_dict if nucsyn_sources_dict else None
+    )
 
     ##########################
     # miscellaneous:
@@ -298,21 +324,35 @@ def parse_binary_c_version_info(version_info_string):
     # TODO: filter a bit more.
 
     misc_dict = {}
-    git_revision = [el for el in cleaned if el.startswith('git revision')]
-    misc_dict['git_revision'] = git_revision[0].split("git revision ")[-1].replace("\"", '')
+    git_revision = [el for el in cleaned if el.startswith("git revision")]
+    misc_dict["git_revision"] = (
+        git_revision[0].split("git revision ")[-1].replace('"', "")
+    )
 
-    git_branch = [el for el in cleaned if el.startswith('git branch')]
-    misc_dict['git_branch'] = git_branch[0].split("git branch ")[-1].replace("\"", '')
+    git_branch = [el for el in cleaned if el.startswith("git branch")]
+    misc_dict["git_branch"] = git_branch[0].split("git branch ")[-1].replace('"', "")
+    cleaned = cleaned - set(git_branch)
 
-    build = [el for el in cleaned if el.startswith('Build')]
-    misc_dict['build'] = build[0].split("Build: ")[-1].replace("\"", '')
+    build = [el for el in cleaned if el.startswith("Build")]
+    misc_dict["build"] = build[0].split("Build: ")[-1].replace('"', "")
+    cleaned = cleaned - set(build)
 
-    email = [el for el in cleaned if el.startswith('Email')]
-    misc_dict['email'] = email[0].split("Email ")[-1].split(',')
+    email = [el for el in cleaned if el.startswith("Email")]
+    misc_dict["email"] = email[0].split("Email ")[-1].split(",")
+    cleaned = cleaned - set(email)
 
-    misc_dict['uncaught'] = cleaned
+    other_items = set([el for el in cleaned if " is " in el])
+    cleaned = cleaned - other_items
 
-    version_info_dict['miscellaneous'] = misc_dict
+    for el in other_items:
+        split = el.split(" is ")
+        key = split[0].strip()
+        val = " is ".join(split[1:]).strip()
+        misc_dict[key] = val
+
+    misc_dict["uncaught"] = "\n".join(cleaned)
+
+    version_info_dict["miscellaneous"] = misc_dict if misc_dict else None
     return version_info_dict
 
 
@@ -643,7 +683,7 @@ def get_help_all(print_help=True):
                 params_dict[param_name] = {
                     "param_name": param_name,
                     "description": param_description,
-                    "rest": ''.join(rest) if rest else '',
+                    "rest": "".join(rest) if rest else "",
                 }
 
             # make section_dict
@@ -709,7 +749,9 @@ def get_help_super(print_help=False, fail_silently=True):
 
                 # Get detailed help info
                 detailed_help = get_help(
-                    parameter_name, print_help=False, fail_silently=fail_silently,
+                    parameter_name,
+                    print_help=False,
+                    fail_silently=fail_silently,
                 )
 
                 if detailed_help:
@@ -931,6 +973,19 @@ class binarycDecoder(json.JSONDecoder):
             return o
 
 
+class BinaryCEncoder(json.JSONEncoder):
+    def default(self, o):
+        print("inarycoij")
+        try:
+            str_repr = str(o)
+        except TypeError:
+            pass
+        else:
+            return str_repr
+        # Let the base class default method raise the TypeError
+        return JSONEncoder.default(self, o)
+
+
 def binaryc_json_serializer(obj):
     """
     Custom serializer for binary_c to use when functions are present in the dictionary
@@ -941,23 +996,17 @@ def binaryc_json_serializer(obj):
 
     if inspect.isfunction(obj):
         return str(obj)
-    return obj
+    else:
+        return obj
+
 
 def handle_ensemble_string_to_json(raw_output):
     """
-    Function that deals with the raw output of the ensemble and 
-    creates a working JSON dictionary out of it. 
+    Function that deals with the raw output of the ensemble and
+    creates a working JSON dictionary out of it.
 
-    It seems that JSON does not like strings that contain dictionaries which have a number as a key, but more specifically it is the fact that binary-c outputs as 5e2 instead of 
-    
-    e.g. with python 3.6.4: json.loads("{5e2: 10}") leads to a JSONDecodeError. To fix this, I do the following:
-
-    with ensemble_output being the literal string of output, and json_output the correct jsonified version of that.
-
-    eval_dict = ast.literal_eval(ensemble_output)
-    json_output =    json.loads(json.dumps(eval_dict), cls=binarycDecoder)
-
-    Note: this does pose somewhat of a risk, since eval'ing something might lead to unexpected behaviour.
+    Having this wrapper makes it easy to
     """
 
-    return json.loads(json.dumps(ast.literal_eval(raw_output)), cls=binarycDecoder)
+    # return json.loads(json.dumps(ast.literal_eval(raw_output)), cls=binarycDecoder)
+    return json.loads(raw_output, cls=binarycDecoder)
